inspect = require 'inspect'

--[[

Ported from the Ejecta demo

https://github.com/phoboslab/ejecta

]]

function radians(degrees)
    return degrees * (math.pi / 180)
end

function rc()
    return math.floor(math.random()*255)
end

function init_curves(n)
    n = math.max(1,n)
    curves = {}
    for i=1,n do
        curves[i] = {
            current = math.random() * 1000,
            inc = math.random() * 0.005 + 0.002,
            color = { rc(), rc(), rc(), rc() }
        }
    end
end

function love.load()
    width, height = love.window.getDimensions()
    w = width
    h = height
    w2 = width / 2
    h2 = height / 2

    init_curves(400)
end

function animate()
    total_line_segments = 0
    for i,v in ipairs(curves) do
        local curve = v
        curve.current = curve.current + curve.inc
        local p = {}
        for j=1,6,2 do
            local a = math.sin( curve.current * (j+3) * 373 * 0.0001)
            local b = math.sin( curve.current * (j+5) * 927 * 0.0002)
            local c = math.sin( curve.current * (j+5) * 573 * 0.0001)
            p[j] = (a * a * b + c * a + b) * w * c + w2
            p[j+1] = (a * b * b + c - a * b *c) * h2 + h2

        end
        love.graphics.setColor(curve.color)
        local bez = love.math.newBezierCurve( unpack(p) )
        local points = bez:render(5)
        total_line_segments = total_line_segments + #points
        love.graphics.line(points)
    end
end

function love.update(dt)
    local new_mouse_x,new_mouse_y = love.mouse.getPosition()
    if new_mouse_x ~= mouse_x then
        init_curves(new_mouse_x*2)
    end
    mouse_x = new_mouse_x
    if new_mouse_y ~= mouse_y then
        love.graphics.setLineWidth(new_mouse_y / 50.0)
    end
    mouse_y = new_mouse_y
end

gp = love.graphics.print

function love.draw()

    animate()

    love.graphics.setColor( {255,255,255,255} )
    gp("FPS: "..love.timer.getFPS(), 10, 10)
    gp("curves:"..#curves, 10, 30)
    gp("total line segments:"..total_line_segments, 10, 50)
    gp("mouse:"..inspect({love.mouse.getPosition()}),10,70)
end
